import Container = FYGE.Container;
import Button = FYGE.Button;
import TextField = FYGE.TextField;
import TEXT_ALIGN = FYGE.TEXT_ALIGN;
import { RES } from "../../../module/RES";
import { Tools } from "../../Tools";
import MouseEvent = FYGE.MouseEvent;
import { showPanel } from "../../../module/ctrls";
import { RulePanel } from "../../panels/RulePanel";
import { TaskPanel } from "../../panels/TaskPanel";
import { sendTbNet, TbNetName } from "../../TaoBaoNet";

export default class MapUI extends Container {
    public energyBox: EnergyBox = null;

    private myPrizeBtn: Button = null;
    private ruleBtn: Button = null;
    private taskBtn: Button = null;

    constructor() {
        super();

        this.initUi();
        this.initEvent();

    }

    private initEvent() {
        this.ruleBtn.addEventListener(MouseEvent.CLICK, this.clickBtn, this);
        this.myPrizeBtn.addEventListener(MouseEvent.CLICK, this.clickBtn, this);
        this.taskBtn.addEventListener(MouseEvent.CLICK, this.clickBtn, this);
    }

    private removeEvent() {
        this.ruleBtn.removeEventListener(MouseEvent.CLICK, this.clickBtn, this);
        this.myPrizeBtn.removeEventListener(MouseEvent.CLICK, this.clickBtn, this);
        this.taskBtn.removeEventListener(MouseEvent.CLICK, this.clickBtn, this);
    }

    private clickBtn(e) {
        switch (e.target) {
            case this.ruleBtn:
                showPanel(RulePanel);
                break;
            case this.myPrizeBtn:
                sendTbNet(
                    TbNetName.navigateTo,
                    { url: '/pages/myPrize/myPrize' },
                    () => {
                    },
                    true
                );
                break;

            case this.taskBtn:
                console.log('任务');
                showPanel(TaskPanel);
                break;
        }
    }

    public destroy() {
        super.destroy();
        this.removeEvent();
    }

    private initUi() {
        // 我的奖品
        this.myPrizeBtn = this.addChild(new Button(RES.getRes('map_prizeBtn.png')));
        this.myPrizeBtn.position.set(617, 213);
        this.myPrizeBtn.anchorX = 133;

        // 规则按钮
        this.ruleBtn = this.addChild(new Button(RES.getRes('map_ruleBtn.png')));
        this.ruleBtn.position.set(0, 213);
        this.ruleBtn.anchorX = 0;

        // 任务按钮
        this.taskBtn = this.addChild(new Button(RES.getRes('map_taskBtn.png')));
        this.taskBtn.position.set(613, 278);

        this.energyBox = this.addChild(new EnergyBox({}));

    }

    public updateUI() {
        this.energyBox.count = Tools.gameData.power;
    }
}

/**
 * 能量
 */
class EnergyBox extends Container {

    private addBtn: Button = null;
    private countLabel: TextField = null;

    private _count = 0;
    public set count(count: number) {
        this._count = count;
        this.countLabel.text = '' + count;
    }

    constructor(data) {
        super();

        this.initUi(data);

    }

    private initUi(data) {
        this.position.set(8, 278);

        this.addChild(Tools.getSprite('map_energyBg.png'));

        this.countLabel = this.addChild(Tools.getText(
            '0', 33, '#000000',
            TEXT_ALIGN.CENTER, 80, 110, 33
        ));

    }

    public destroy() {
        super.destroy();
    }
}
